/*
   DScope Application

   Copyright (C) 2008,2009:
         Daniel Roggen, droggen@gmail.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "mainwindow.h"
#include "ui_mainwindow.h"
#include <QMessageBox>
#include <QFileDialog>
#include <vector>
#include <fstream>
#include <sstream>
#include <iostream>
#include <iterator>

#include "cio.h"
#include "precisetimer.h"

#include "portwindow.h"
#include "helpwindow.h"

#include <qextserialenumerator.h>
#include <qextserialport.h>

/*
 Example format:
 <Calibrated;5 6 7;xscale=1;yscale=-3300 3300 ;color=ff0000 ff00 ff><Raw;2 3 4;xscale=1;yscale=0 1024 ;color=ff0000 ff00 ff>
*/



MainWindow::MainWindow(QWidget *parent) :
   QMainWindow(parent),
   ui(new Ui::MainWindow),
   fp(string(""))
{
   ui->setupUi(this);

#ifdef DEVELMODE
   ConsoleInit();
   printf("Starting up\n");
#endif




   totreceiveddata=0;

   // Set the NaN replacement value
   nanvalue=0;
   ui->uiNanValue->setValue(nanvalue);

   // Set how much past data we keep
   buffersize=100000;
   ui->uiBufferSize->setValue(buffersize);


   // Initialize the scopes
   dscopes = new DScopesQTWidget(0,0,640,480,false);
   ui->mainVerticalLayout->addWidget(dscopes);
   resize(800, 640);

   // Initialize the afterglow to reflect the scope
   ui->uiAfterGlow->setCheckState(Qt::Unchecked);

   // Initialize the refreshrate
   ui->uiRefreshRate->setValue(10);
   on_uiRefreshRate_valueChanged(10);



   tcpSocket = new QTcpSocket(this);
   port=0;
   binary=false;		// Text mode by default

   //printf("Read buffer size: %d\n",tcpSocket->readBufferSize());

   connect(tcpSocket, SIGNAL(readyRead()), this, SLOT(tcpRead()));
   connect(tcpSocket, SIGNAL(error(QAbstractSocket::SocketError)),this, SLOT(displayError(QAbstractSocket::SocketError)));


   ui->uileHostPort->setText(QString("ser:com4:19200"));

   pt=0;
   lastplot=0;


   fp.Status();

   Connected=0;


   time_received_last=0;
   time_received_delta=0;
   time_displayed_last=PreciseTimer::QueryTimer();
   time_displayed_delta=0;

   datarateLabel=new QLabel(statusBar());
   statusBar()->addWidget(datarateLabel);
   displayrateLabel=new QLabel(statusBar());
   statusBar()->addWidget(displayrateLabel);

   time_received_delta_vector.resize(100,0.0);

   QStringList arg = qApp->arguments();
   if(arg.contains("-f"))
   {
      int idx=arg.indexOf("-f");
      if(idx<arg.size()-1)
         loadConfiguration(arg.at(idx+1));
      else
         QMessageBox::critical(this, "Command line error", "Missing configuration file name to load");
   }
   if(arg.contains("-c"))
   {
      on_uipbConnect_clicked();
   }
}

MainWindow::~MainWindow()
{
   delete ui;
}

void MainWindow::SaveData()
{
   QString t="Save data ";
   QString fileName = QFileDialog::getSaveFileName(this, t,QString(),"Text (*.txt)");
   if(!fileName.isNull())
   {
      QFile file(fileName);
      if (file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate))
      {
         // Stream writer...
         QTextStream out(&file);

         int i1,i2;
         i2=alldata[0].size();
         if(totreceiveddata<=i2)
            i1=i2-totreceiveddata;
         else
            i1=0;
         for(int i=i1;i<i2;i++)
         {
            for(unsigned j=0;j<alldata.size();j++)
               out << alldata[j][i] << " ";
            out << endl;
         }
        file.close();
      }
      else
      {
         QMessageBox::critical(this,t+"error", "Cannot write to file");
      }
   }
}


void MainWindow::Plot()
{
   double t1,t2;
   //t1=PreciseTimer::QueryTimer();
   // Plot scopes
   if(sd.scopedefinition.size()==0)
   {
      for(unsigned i=0;i<alldata.size();i++)
         dscopes->operator[](handles[i])->Plot(alldata[i],0xffffffff);
   }
   else
   {
      // Iterate all scopes
      for(unsigned s=0;s<sd.scopedefinition.size();s++)
      {
         // Iterate all traces of the scope
         vector<vector<int> *> vd;		// data for the traces in this scope
         vector<unsigned> vc;			// colors  of the traces
         for(unsigned t=0;t<sd.scopedefinition[s].traces.size();t++)
         {
            // If trace available
            if(sd.scopedefinition[s].traces[t]<alldata.size())
            {
               vd.push_back(&alldata[sd.scopedefinition[s].traces[t]]);
               if(sd.scopedefinition[s].colors.size()!=0)
                  vc.push_back(sd.scopedefinition[s].colors[t]);
               else
                  vc.push_back(0xffffffff);
            }
            dscopes->operator[](handles[s])->Plot(vd,vc);
         }
      }
   }
   /*t2=PreciseTimer::QueryTimer();
   pt = 0.9*pt+0.1*(t2-t1);
   printf("plot time: %.5lf\n",pt);*/
//	dscopes->Render();
}
void MainWindow::timerEvent(QTimerEvent *event)
{
   double f=0.9;
   double t = PreciseTimer::QueryTimer();
   time_displayed_delta = f*time_displayed_delta + (1.0-f)*(t-time_displayed_last);
   time_displayed_last=t;
   QString s;
   s.sprintf("Display rate: %3.0lf Hz",1.0/time_displayed_delta);
   displayrateLabel->setText(s);

   Plot();
   dscopes->Render();
   dscopes->repaint();

}



void MainWindow::on_uipbConnect_clicked()
{
   if(Connected!=0)
   {
      // If connected, disconnect.
      if(Connected==1)
      {
         // TCP disconnect
         if( tcpSocket->state() != QAbstractSocket::UnconnectedState )
         {
            tcpSocket->disconnectFromHost();
            if(tcpSocket->state() != QAbstractSocket::UnconnectedState)
            {
               if(!tcpSocket->waitForDisconnected())
               {
                  QMessageBox::critical(this, "Disconnect failed", "Could not disconnect from host\n");
                  return;
               }
            }
         }
      }
      else
      {
         // Serial disconnect
         printf("Before close\n");
         port->close();
         printf("After close\n");
         //if(port) { delete port; port=0; }
      }
      Connected=0;
      // Change the button text.
      ui->uipbConnect->setText("&Connect");
   }
   else
   {
      totreceiveddata=0;
      // If not connected, connect.
      // tcp:host:port
      // ser:com:speed
      // Regexp: spaces (tcp or ser) spaces colon spaces (host/com) spaces colon spaces (port/speed) spaces
      // Capture 0: entire string. Capture 1: tcp|ser. Capture 2: host|com. Capture 3: port|speed
      QRegExp v("\\s*(tcp|ser)\\s*:\\s*(.+)\\s*:\\s*(\\d+)\\s*");
      bool expok=true;
      bool tcp;
      int index=v.indexIn(ui->uileHostPort->text().toLower());
      if(v.cap(1).compare("tcp")==0)
      {
         tcp=true;
      }
      else
      {
         if(v.cap(1).compare("ser")==0)
            tcp=false;
         else
            expok=false;
      }

      bool cap3ok;
      unsigned cap3 = v.cap(3).toUInt(&cap3ok);
      if(index==-1 || cap3ok==false || expok==false)
      {
         QMessageBox::critical(this, "Invalid device specification", "Device is specified as follows:\nser:com:speed\ntcp:host:port\nExample: \"127.0.0.1:50008\"");
         return;
      }
      printf("Connect as: %d %s %d\n",tcp,v.cap(2).toStdString().c_str(),cap3);

      if(tcp)
      {
         // Connect to host
         tcpSocket->connectToHost(v.cap(2),cap3);
         if (!tcpSocket->waitForConnected(2000))
         {
            QMessageBox::critical(this, "Connection failed", "Connection to host failed. Check host:port settings.\n");
            return;
         }
         Connected=1;
      }
      else
      {
         if(cap3!=110&& cap3!=300&& cap3!=600&& cap3!=1200&& cap3!=2400&& cap3!=4800&& cap3!=9600&& cap3!=14400&&
          cap3!=19200&& cap3!=38400&& cap3!=57600&&  cap3!=115200)
         {
            QMessageBox::critical(this, "Invalid parameter", "Invalid baud rate.");
            return;
         }
         if(port) { delete port; port=0; }
         printf("port to open: %s\n",v.cap(2).toStdString().c_str());
         // This will be windows only:
#ifdef WIN32
         string portstring=string("\\\\.\\\\") + v.cap(2).toStdString();
#else
         string portstring=v.cap(2).toStdString();
#endif
         printf("port to open: %s\n",portstring.c_str());
         //port = new QextSerialPort(portstring.c_str(),QextSerialBase::EventDriven);
         port = new QextSerialPort(portstring.c_str(),QextSerialPort::EventDriven);
         //port = new QextSerialPort(v.cap(2).toStdString().c_str(),QextSerialBase::EventDriven);
         printf("Before calling open\n");
         port->open(QIODevice::ReadOnly);
         printf("After calling open\n");
         if(port->isOpen())
         {
            switch(cap3)
            {
               case 110:
                  port->setBaudRate(BAUD110);
                  break;
               case 300:
                  port->setBaudRate(BAUD300);
                  break;
               case 600:
                  port->setBaudRate(BAUD600);
                  break;
               case 1200:
                  port->setBaudRate(BAUD1200);
                  break;
               case 2400:
                  port->setBaudRate(BAUD2400);
                  break;
               case 4800:
                  port->setBaudRate(BAUD4800);
                  break;
               case 9600:
                  port->setBaudRate(BAUD9600);
                  break;
               case 14400:
                  port->setBaudRate(BAUD14400);
                  break;
               case 19200:
                  port->setBaudRate(BAUD19200);
                  break;
               case 38400:
                  port->setBaudRate(BAUD38400);
                  break;
               case 57600:
                  port->setBaudRate(BAUD57600);
                  break;
               default:
                  port->setBaudRate(BAUD115200);
               }
            port->setDataBits(DATA_8);
            port->setFlowControl(FLOW_OFF);
            port->setParity(PAR_NONE);
            port->setStopBits(STOP_1);
            //ps.Timeout_Millisec=500;
            port->setTimeout(500);
            connect(port,SIGNAL(readyRead()),this,SLOT(serialRead()));
            connect(port,SIGNAL(aboutToClose()),this,SLOT(aboutToClose()));
            connect(port,SIGNAL(dsrChanged(bool)),this,SLOT(dsrChanged(bool)));
            Connected=2;
         }
         else
         {
            QMessageBox::critical(this, "Connection failed", "Cannot open serial device. Check port:speed settings.\n");
            return;
         }
      }
      time_received_last=PreciseTimer::QueryTimer();
      ui->uipbConnect->setText("Dis&connect");


   }
}
void MainWindow::aboutToClose()
{
        printf("MainWindow::aboutToClose\n");
}

void MainWindow::DeleteAllScopes()
{
   while(handles.size())
   {
      dscopes->RemoveScope(*(handles.end()-1));
      handles.erase(handles.end()-1);
   }
}
void MainWindow::AddScopes(unsigned n)
{
   handles = dscopes->AddScopes(n);
}


/**
  \brief Do something with NaNs - typically set a constant value in the data buffer for display purposes
**/
void MainWindow::treatNaN(vector<int> &linedata,std::vector<bool> &linedatanan)
{
   for(unsigned i=0;i<linedata.size();i++)
      if(linedatanan[i])
         linedata[i]=nanvalue;
}

void MainWindow::receivedData(vector<int> &linedata,std::vector<bool> &linedatanan)
{
   // Handle the NaN
   treatNaN(linedata,linedatanan);


   // Keep a buffer of the last

   double f=0.99;
   double t = PreciseTimer::QueryTimer();
   if(linedata.size())
   {

      time_received_delta_vector.push_back(t-time_received_last);
      time_received_delta = f*time_received_delta + (1.0-f)*(t-time_received_last);

      time_received_delta_vector.erase(time_received_delta_vector.begin());
      double tt=0;
      for(int i=0;i<time_received_delta_vector.size();i++)
         tt+=time_received_delta_vector[i];
      tt/=(double)time_received_delta_vector.size();

      QString s;
      //s.sprintf("Data rate: %3.1lf Hz  %3.1lf Hz",1.0/time_received_delta,1.0/tt);
      s.sprintf("Data rate: %3.0lf Hz",1.0/tt);
      //s.sprintf("Data rate: %3.0lf Hz",1.0/time_received_delta);
      datarateLabel->setText(s);


      time_received_last=t;


   }





   unsigned newsize = linedata.size();
   unsigned oldsize = alldata.size();
   // Check for sensor changes
   if(oldsize!=newsize)
   {
      printf("Changing from %d to %d\n",oldsize,newsize);

      // Resize the data vector according to the new number of sensors
      // Since we cannot identify sensors we should in principle erase all past data to avoid confusion - we don't.
      if(newsize<oldsize)
      {
         for(unsigned i=0;i<oldsize-newsize;i++)
            alldata.erase(alldata.end()-1);
      }
      else
      {
         for(unsigned i=0;i<newsize-oldsize;i++)
            //alldata.push_back(vector<int> (buffersize,0));
            alldata.push_back(vector<int> (buffersize,nanvalue));
      }
      // Adjust scope number
      if(sd.scopedefinition.size()==0)
      {
         DeleteAllScopes();
         AddScopes(newsize);
      }
   }
   // Shift the data in the data buffer
   for(unsigned i=0;i<alldata.size();i++)
   {
      // Erase old data
      alldata[i].erase(alldata[i].begin());
      // Store new data
      alldata[i].push_back(linedata[i]);
   }
   /*if(PreciseTimer::QueryTimer()-lastplot>refresh)
   {
      Plot();
      dscopes->Render();
      dscopes->repaint();
      lastplot = PreciseTimer::QueryTimer();
   }*/

   totreceiveddata++;
}
void MainWindow::displayError(QAbstractSocket::SocketError)
{
   printf("Socket error\n");
}


void MainWindow::on_uiRefreshRate_valueChanged(int i)
{
   killTimer(timer);
   timer=startTimer(1000/i);
   refresh=1.0/(double)i;
}
void MainWindow::on_uiNanValue_valueChanged(int i)
{
   nanvalue = i;
   printf("changed nan to %d\n",i);
}

void MainWindow::on_uiBufferSize_valueChanged(int i)
{
   buffersize=i;
   // We cut/expand alldata, but only if there are some data in alldata
   if(alldata.size()==0)
      return;
   // Get the how much is the buffer size currently
   unsigned datasize = alldata[0].size();
   // Adjust the buffer size
   printf("Adjust buffer from %d to %d\n",datasize,buffersize);
   for(unsigned c=0;c<alldata.size();c++)
   {
      // Need to cut datasize-buffersize data
      if(buffersize<datasize)
         alldata[c].erase(alldata[c].begin(),alldata[c].begin()+datasize-buffersize);
      // Need to prepend buffersize-datasize data
      if(buffersize>datasize)
         alldata[c].insert(alldata[c].begin(),buffersize-datasize,nanvalue);
   }
}


void MainWindow::on_uiAfterGlow_stateChanged(int state)
{
   dscopes->SetAlpha(state==Qt::Checked);
}


void MainWindow::on_uipbApplyDisplay_clicked()
{
   sd.scopedefinition.clear();
   //int rv = ParseLayoutString(uileDisplayFormat->text().toStdString(),sd);
   int rv = ParseLayoutString(ui->uiteDisplayFormat->toPlainText().toStdString(),sd);
   if(rv!=0)
   {
      printf("Error in layout string\n");
      QMessageBox::information(this, "Invalid display format string","Check help for more information");
      return;
   }

   // Erase all the scopes
   DeleteAllScopes();
   // Hardcore init of the number of scopes
   if(sd.scopedefinition.size()!=0)
   {

      handles=dscopes->AddScopes(sd.scopedefinition.size());
      for(unsigned i=0;i<sd.scopedefinition.size();i++)
      {
         dscopes->operator[](handles[i])->SetTitle(sd.scopedefinition[i].title);
         if(sd.scopedefinition[i].yauto)
            dscopes->operator[](handles[i])->SetVAuto();
         else
            dscopes->operator[](handles[i])->SetVRange(sd.scopedefinition[i].yscale[0],sd.scopedefinition[i].yscale[1]);
         dscopes->operator[](handles[i])->HZoom(sd.scopedefinition[i].xscale);
      }
      vector<int> v(1,0);
      for(unsigned i=0;i<sd.scopedefinition.size();i++)
         dscopes->operator[](handles[i])->Plot(v);
   }
   else	// Variable number of scopes: adjust to the number of data last received
   {
      handles = dscopes->AddScopes(alldata.size());
   }
   dscopes->Render();
   dscopes->repaint();
}
void MainWindow::on_uipbApplyBinary_clicked()
{
   // DX3 reader: DX3;cc-s-s-s-s-s-s

   // Test the binary string.
   string binaryformat = ui->uiteBinaryFormat->toPlainText().trimmed().toStdString();
   if(binaryformat.size()==0)
   {
      binary=false;
   }
   else
   {
      binary=true;
      fp = FrameParser3(binaryformat);
      if(!fp.IsValid())
         QMessageBox::critical(this, "Invalid binary format", "Binary format specification is: \"<header>;<[-]c|s|S|i|I|b<num>|B<num>><[-]c|s|S|i|I|b<num>|B<num>>...[;<x|f|F>]\"");
   }
}


void MainWindow::tcpRead()
{
   //printf("tcpRead bytes available: %d\n",tcpSocket->bytesAvailable());
   if(binary)
   {
      QByteArray ba=tcpSocket->readAll();
      BinaryChunckDecodeRead(ba);
   }
   else
   {
      QTextStream in(tcpSocket);
      TextChunckDecodeRead(in);
   }
}
void MainWindow::serialRead()
{
   if(binary)
   {
      QByteArray ba=port->readAll();
      BinaryChunckDecodeRead(ba);
   }
   else
   {
      QTextStream in(port);
      TextChunckDecodeRead(in);
   }
}
void MainWindow::TextChunckDecodeRead(QTextStream &in)
{
   while(1)
   {
      QString qstr = in.readLine();
      if(qstr.isNull())
         break;

      // Check the length of the string
      //printf("%d %d\n",qstr.size(),strlen(qstr.toAscii().data()));

      // Generate synthetic string
      //qstr =QString("   3      10 \t 1213 , -31239 ; NaN 3891 NaN NaN 12902 3.1415 -2.7 10.00");

      QRegExp rx(QString("[ ;,\t]"));                             // Match any separator
      QStringList sl = qstr.split(rx,QString::SkipEmptyParts);		// Split on separator, skip empty parts. We don't want multiple spaces or tabs to be interpreted as multiple strings (
      vector<int> linedata(sl.size());
      vector<bool> linedatanan(sl.size());
      for(int i=0;i<sl.size();i++)			// Convert string list to int vector
      {
         // Check for NaN
         if(QString("NaN").compare(sl[i],Qt::CaseInsensitive)==0)
         {
            linedata[i]=0;
            linedatanan[i]=true;
         }
         else
         {
            // Allow floats, but cast them to int
            bool ok;
            double v = sl[i].toDouble(&ok);
            int vi = (int)v;
            if( ok )
            {
               linedata[i] = vi;
               linedatanan[i] = false;
            }
            else
            {
               linedata[i] = 0;
               linedatanan[i] = true;
            }
         }
      }
      /*
      for(int i=0;i<sl.size();i++)
      {
         if(linedatanan[i]==false)
            printf("%d ",linedata[i]);
         else
            printf("- ");
      }
      printf("\n");
      */




      receivedData(linedata,linedatanan);
   }
}
void MainWindow::BinaryChunckDecodeRead(QByteArray &ba)
{
   vector<vector<int> > v = fp.Parser(ba.data(),ba.size());
   if(v.size())
   {
      for(int i=0;i<v.size();i++)
      {
         std::vector<bool> datanan(v[i].size(),false);
         receivedData(v[i],datanan);
      }
   }
}
void MainWindow::dsrChanged(bool s)
{
   printf("dsrChanged status: %d\n",s);
}

/******************************************************************************
  on_actionSaveConfiguration_triggered
*******************************************************************************
Save the configuration in an XML file
******************************************************************************/
void MainWindow::on_actionSaveConfiguration_triggered()
{
   QString fileName = QFileDialog::getSaveFileName(this, "Save Configuration",QString(),"Configuration (*.xml)");
   if(!fileName.isNull())
   {
      QFile file(fileName);
      if (file.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Truncate))
      {
         QXmlStreamWriter xml(&file);
         xml.writeStartDocument();
         xml.writeStartElement("Configuration") ;
         xml.writeAttribute("DisplayFormat",ui->uiteDisplayFormat->toPlainText());
         xml.writeAttribute("BinaryFormat",ui->uiteBinaryFormat->toPlainText());
         xml.writeAttribute("InputDevice",ui->uileHostPort->text());
         xml.writeAttribute("BufferSize",ui->uiBufferSize->text());
         xml.writeAttribute("NanValue",QString("%1").arg(nanvalue));
         xml.writeEndElement();
         xml.writeEndDocument();
         file.close();
      }
      else
      {
         QMessageBox::critical(this, "Save configuration error", "Cannot write to file");
      }
   }
}
void MainWindow::on_actionLoadConfiguration_triggered()
{
   QString fileName = QFileDialog::getOpenFileName(this, "Load Configuration",QString(),"Configuration (*.xml)");
   if(!fileName.isNull())
      loadConfiguration(fileName);
}

/******************************************************************************
  loadConfiguration
*******************************************************************************
Load the configuration from an XML file.
If there are missing parameters, the fields remain unchanged.
******************************************************************************/
void MainWindow::loadConfiguration(QString fileName)
{
   QFile file(fileName);
   if (file.open(QIODevice::ReadOnly | QIODevice::Text))
   {
      QXmlStreamReader xml(&file);
      while (!xml.atEnd())
      {
         QXmlStreamReader::TokenType type = xml.readNext();
         //printf("readNext type: %d\n",type);
         if(type==QXmlStreamReader::StartElement)
         {
            //printf("name: %s\n",xml.name().toString().toStdString().c_str());
            //printf("namespaceUri: %s\n",xml.namespaceUri().toString().toStdString().c_str());
            if(xml.name().toString().compare("Configuration")==0)
            {
               QXmlStreamAttributes at = xml.attributes();
               QString df = at.value("DisplayFormat").toString();
               QString bf = at.value("BinaryFormat").toString();
               QString id = at.value("InputDevice").toString();
               QString bs = at.value("BufferSize").toString();
               QString nv = at.value("NanValue").toString();
               if(!df.isNull())
                  ui->uiteDisplayFormat->setPlainText(df);
               if(!bf.isNull())
                  ui->uiteBinaryFormat->setPlainText(bf);
               if(!id.isNull())
                  ui->uileHostPort->setText(id);
               // Buffer size
               if(!bs.isNull())
               {
                  bool ok;
                  unsigned bs2 = bs.toUInt(&ok);
                  if(ok)
                  {
                     if(bs2<ui->uiBufferSize->minimum())
                        bs2=ui->uiBufferSize->minimum();
                     if(bs2>ui->uiBufferSize->maximum())
                        bs2=ui->uiBufferSize->maximum();
                     // The onValueChanged event will change buffersize if needed
                     ui->uiBufferSize->setValue(bs2);
                  }
               }
               // Nan value
               if(!nv.isNull())
               {
                  bool ok;
                  int nv2 = nv.toInt(&ok);
                  if(ok)
                  {
                     if(nv2<ui->uiNanValue->minimum())
                        nv2=ui->uiNanValue->minimum();
                     if(nv2>ui->uiNanValue->maximum())
                        nv2=ui->uiNanValue->maximum();
                     // The onValueChanged event will change nanvalue if needed
                     ui->uiNanValue->setValue(nv2);
                  }
               }
               on_uipbApplyDisplay_clicked();
               on_uipbApplyBinary_clicked();
            }
         }
      }
      file.close();
   }
   else
   {
      QMessageBox::critical(this, "Load configuration error", "Cannot read from file");
   }
}


void MainWindow::on_actionAbout_triggered()
{
   QMessageBox::about(this, "About",
   "<p><b>DScope</b> - QT Version</p>\n"
   "<p>Version 1.07</p>"
   "<p>(c) 2007-2011 Daniel Roggen</p>");

}
void MainWindow::on_actionListSerialPorts_triggered()
{
   PortWindow dialog(this);
   int ret = dialog.exec();
   if (ret == QDialog::Rejected)
      return;
}
void MainWindow::on_actionHowto_triggered()
{

   QFile file(":/howto.html");
   file.open(QIODevice::ReadOnly | QIODevice::Text);
   QByteArray filedata = file.readAll();
   HelpWindow dialog(QString(filedata),this);
   dialog.exec();
}

void MainWindow::on_pushButton_ClearData_clicked()
{
   totreceiveddata=0;
   alldata.clear();
   if(sd.scopedefinition.size()==0)
      DeleteAllScopes();
}

void MainWindow::on_pushButton_SaveData_clicked()
{
   if(totreceiveddata==0)
   {
      QMessageBox::information(this,"Save data","No data has been collected - connect to a device first");
   }
   SaveData();

}



